%
%           Programa Automatico de calculo XFEM para FEAP
% ============================================================
%  Crea los archivos archivos 'input' automaticamente para que
%  sean procesados por el generador de archivos FileGen.
%  Calcula en FEAP estos archivos y almacena los 
%  resultados en carpetas 'caso i' en cada ciclo
%   
%=============================================================




clear all

deltaa=0.1;
nincrpropag=11; %Nmero de incrementos de propagacin

meshgenera=1; %Flag para generar malla o importarla 
             %  0 =>no genera, 1 => s genera
numoverlay=100000; %Para aadir a la numeracin de elem overlay



for propag=1:nincrpropag


% Aqu empieza el autoXAbq clsico, como estaba antes:

% Para que no d la tabarra con el mensaje de error
warning off MATLAB:MKDIR:DirectoryExists

% Definicion de parametros del problema
%---------------------------------------

ancho=6.0;            %Ancho de la placa
largo=2.0;           %Largo de la placa

Carga=[0 100];          %NO hace falta para ABAQUS



%Refinamiento EG en nodo
NX=[120  10	 20	  40]';       %Numero de elementos en x      
NY=[41   40	 80	 160]';      %Numero de elementos en y


% Dibuja malla con gdl enriquecidos
dibujomalla=1;
           % 1=Dibuja


%Dibujar deformada
deformada=0;
escala=0;  
            % 0=Dibuja con escala automatica cada iteracion
            % 1=Dibuja con la primer escala calculada
            % 2=Dibuja con una escala fija siempre
escalafija=1000; 

tensiones=0;
            % 0=No dibuja las tensiones
            % 1=Dibuja tension 1
            % 2=Dibuja tension 2 ...

%Generacion de puntos de grieta
%------------------------------
%   (El primer punto es el origen, los siguientes son diferentes
%   extremos de grieta 

if propag==1

 mkdir('../','Result'); %Generacin del directorio Result   
    
 XYGrieta(1,:)= [6.0  4.0];
 XYGrieta(2,:)= [largo/2+0.07 largo/2+0.07];

 fid = fopen(['../Result\','XYGrPropag.prn'],'w');
 fprintf(fid,'%3.15f   %3.15f\n',XYGrieta);    
 fclose(fid);

else

load ../Result/XYGrPropag.prn
XYGr=XYGrPropag
npg=size(XYGr,1)
 % Puesto a pelo para dibujar el ltimo estado (hacia arriba)
%thetaMCS=-3.192196501980800


% Inclinacin ltimo tramo de grieta:

 long=sqrt((XYGr(npg,1)-XYGr(npg-1,1))^2+(XYGr(npg,2)-XYGr(npg-1,2))^2);
 cosalfa=(XYGr(npg,1)-XYGr(npg-1,1))/long;
 sinalfa=(XYGr(npg,2)-XYGr(npg-1,2))/long;
 
% Matriz transformacin de globales a locales:
 T=[cosalfa sinalfa; -sinalfa cosalfa];
    
% Clculo extensin de grieta en sist local con el ngulo criterio propag: 
 deltaXYlocal(1)=deltaa*cos(thetaMCS*pi/180);
 deltaXYlocal(2)=deltaa*sin(thetaMCS*pi/180);
 
% Paso de extensiones en sistema local a sistema global: 
 deltaXYglobal= T'*deltaXYlocal';

% Nuevos puntos finales de grieta en sistema global: 
 nuevoXgr=XYGr(npg,1)+deltaXYglobal(1);
 nuevoYgr=XYGr(npg,2)+deltaXYglobal(2);
 fid = fopen(['../Result\','XYGrPropag.prn'],'a');
 fprintf(fid,'%3.15f   %3.15f\n',nuevoXgr,nuevoYgr);    
 fclose(fid);

 fid = fopen(['../Result\','XYGrPropag.prn'],'r');
 XYGrieta=fscanf(fid,'%g %g\n',[2,inf]);    
 fclose(fid);

end

npg=size(XYGrieta,2);
npx=size(NX); %nmero de mallas preparadas


%Solucion para diferentes extremos de grieta parametrizados:
%------------------------------
%for i=2:npg

             
for i=1:1 %Esto de momento lo dejo solo para la primera y nica malla
   

    %Escritura del archivo input
    XYGr=XYGrieta';

    fid = fopen('input','w');
    fprintf(fid,'NelmX =\n');
    
    %fprintf(fid,'%3.0f\n',NX(1));
    fprintf(fid,'%3.0f\n',NX(i));
    
    fprintf(fid,'Nelmy =\n');
    
    %fprintf(fid,'%3.0f\n',NY(1));
    fprintf(fid,'%3.0f\n',NY(i));    
    
    fprintf(fid,'w =\n');
    fprintf(fid,'%3.15f\n',ancho);
    fprintf(fid,'l =\n');
    fprintf(fid,'%3.15f\n',largo);
    fprintf(fid,'CargasL =\n');
    fprintf(fid,'%3.15f %3.15f\n',Carga);
    fprintf(fid,'XYG =\n');
    fprintf(fid,'%3.15f %3.15f\n',XYGr');    
    fclose(fid);

 
%   Llamada a la rutina filegen

    FileGenAbq(dibujomalla,i,propag,meshgenera,numoverlay);

%    break
    
    %Ejecucion de FEAP con los archivos generados
%    dos('../OBJ\feap\Debug\feap.exe -iixfem');
    
    %Almacenamiento de los resultados en directorios separados
    
 
    caso=['caso', int2str(i-1),'/ficheros'];
    mkdir('../Workdir',caso);

    debug=['caso', int2str(i-1),'/debug'];
    mkdir('../Workdir',debug);
    postproc=['caso', int2str(i-1),'/postproc'];
    postprocMSDOS=['caso', int2str(i-1),'\postproc'];
    mkdir('../Workdir',postproc);
    

    casosinficheros=['caso', int2str(i-1)];

    
    %Dibujo de la deformada
    if deformada==1 
       [factor,escalafija]= deformadaX(escala,escalafija,(i-1));
    end
    
    if tensiones~=0
        dibtension(tensiones);
    end    

%%% Comento lo que No hace falta para ABAQUS    
    

    movefile('infoXFEM',['../Workdir\',caso]);
%    movefile('ixfem',['../Workdir\',caso]);
%    movefile('oxfem',['../Workdir\',caso]);    
    movefile('input',['../Workdir\',caso]);  
%    movefile('BCX',['../Workdir\',caso]);
%    movefile('FORCEX',['../Workdir\',caso]);
    movefile('Top',['../Workdir\',caso]);
    movefile('XY',['../Workdir\',caso]);
%    movefile('Sol',['../Workdir\',caso]);
%    movefile('NodeStress',['../Workdir\',caso]);
%    movefile('SubEleStress',['../Workdir\',caso]);

    movefile('TopSinX',['../Workdir\',caso]);
    movefile('TopConX',['../Workdir\',caso]);
    movefile('TopConXTipoX',['../Workdir\',caso]);
    movefile('TopConXoverlay',['../Workdir\',caso]);

    movefile('FilaElemX',['../Workdir\',caso]);
    movefile('FilaNodoX',['../Workdir\',caso]);


    movefile('SETladoizq',['../Workdir\',caso]);
    movefile('SETladoder',['../Workdir\',caso]);
    movefile('SETladoarr',['../Workdir\',caso]);
    movefile('SETladoaba',['../Workdir\',caso]);
    movefile('SETnodoabaizq',['../Workdir\',caso]);
    movefile('SETnodoabadch',['../Workdir\',caso]);
    movefile('SETnodoarrdch',['../Workdir\',caso]);
    movefile('SETnodoarrizq',['../Workdir\',caso]);
    movefile('SETnodocen',['../Workdir\',caso]);
    movefile('SETigrieta',['../Workdir\',caso]);
    movefile('SETfgrieta',['../Workdir\',caso]);
    movefile('SETelemarr',['../Workdir\',caso]);
    movefile('SETelemaba',['../Workdir\',caso]);

    movefile('SETNodoX2gdl',['../Workdir\',caso]);
    movefile('SETNodoX4gdl',['../Workdir\',caso]);
    movefile('SETNodoX10gdl',['../Workdir\',caso]);
 
 %  Por si hay que imponer algunas cdc sobre nodos enriquecidos
 %   movefile('SETrestrgdl3',['../Workdir\',caso]);
    
    movefile('GGinfoX',['../Workdir\',caso]);
    movefile('GGXYG',['../Workdir\',caso]);
    movefile('GGnodoX',['../Workdir\',caso]);
    movefile('GGelemX',['../Workdir\',caso]);

    copyfile('../InputABQ\abaCAE.bat',['../Workdir\',casosinficheros]);
    copyfile('../InputABQ\abaXFEM.bat',['../Workdir\',casosinficheros]);
    copyfile('../InputABQ\grXFEM.inp',['../Workdir\',casosinficheros]);
    copyfile('../InputABQ\leerdatos.txt',['../Workdir\',postproc]);

 
      
    %Ejecucion de ABAQUS con los archivos generados

    rutaabaXFEMbat=['cd ..\Workdir\',casosinficheros];
    fid = fopen('runABQ.bat','w');
      fprintf(fid,'%s\n',rutaabaXFEMbat);
      fprintf(fid,'%s','abaXFEM.bat');
    fclose(fid);
    dos('runABQ.bat');
    
%    Para borrar ficheros de ABQ que ocupan y no uso    
         
    fid = fopen('rundelABQ.bat','w');
      fprintf(fid,'%s\n',rutaabaXFEMbat);
      fprintf(fid,'%s\n','del grXFEM.stt');
      fprintf(fid,'%s\n','del grXFEM.res');
      fprintf(fid,'%s\n','del grXFEM.mdl');
      fprintf(fid,'%s','del grXFEM.prt');
    fclose(fid);
    dos('rundelABQ.bat');

%    No se por que, lo que sigue no va. Asi que he hecho lo de arriba
%    dos('../Workdir\caso0\abaXFEM.bat');
%    dos('../OBJ\feap\Debug\feap.exe -iixfem');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    PARA POSTPROCESAR EL VALOR DE IJ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
    
% Generacin ficheros contienen el inicio y final del ltimo tramo de grieta, para calcular
% cos directores en la rutina IJarea

%  Punto inicio ltimo tramo:
    fid = fopen(['../Workdir\',postproc,'\coorIniUltimTramo'],'w');
      fprintf(fid,'%22.15e  %22.15e  %22.15e  %4g',XYGr(npg-1,1),XYGr(npg-1,2),0,9999);
    fclose(fid);

%  Punto final ltimo tramo:
    fid = fopen(['../Workdir\',postproc,'\coorFinUltimTramo'],'w');
      fprintf(fid,'%22.15e  %22.15e  %22.15e  %4g',XYGr(npg,1),XYGr(npg,2),0,9999);
    fclose(fid);

    %copyfile(['../Workdir\',casosinficheros,'\grXFEM.fil'],['../Workdir\',postproc]);  
    movefile(['../Workdir\',casosinficheros,'\grXFEM.fil'],['../Workdir\',postproc]);  
    
    copyfile(['../../../../ULTIMOIJAREA\','ijarea.exe'],['../Workdir\',postproc]);
    copyfile(['../../../../ULTIMOIJAREA\','ijareaEXE.bat'],['../Workdir\',postproc]);
    copyfile(['../../../../ULTIMOIJAREA\','delfichijarea.bat'],['../Workdir\',postproc]);

    rutaIJAREA=['cd ..\Workdir\',postprocMSDOS];
    fid = fopen('runIJAREA.bat','w');
      fprintf(fid,'%s\n',rutaIJAREA);
      fprintf(fid,'%s\n','ijareaEXE.bat');
    fclose(fid);
     a='Paso por 1'
    dos('runIJAREA.bat');
    
    a='Paso por 2'
    
    fid = fopen('rundelfichIJAREA.bat','w');
      fprintf(fid,'%s\n',rutaIJAREA);
      fprintf(fid,'%s','delfichijarea.bat');
    fclose(fid);
    dos('rundelfichIJAREA.bat');
    
    
    
    
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    PARA ESCRIBIR FICHERO COMPACTO DE RESULTADOS PARA IJ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if ((i==1)&(propag==1))
     fid1= fopen(['../Workdir\',postproc,'\J_RESULT.prn'],'r');
     fid2 = fopen(['../Result\','\RESULT.prn'],'w');    
     fprintf(fid2,'%s\n','Tramo  XfinGr             YfinGr              J FEM               J XFEM              J Suma               KI                  KII               Angulo local MCS' );    
    else
      fid1= fopen(['../Workdir\',postproc,'\J_RESULT.prn'],'r');
      fid2 = fopen(['../Result\','\RESULT.prn'],'a');
    end

    for jj=1:29
       tline = fgetl(fid1); 
    end
    basu=fscanf(fid1,'%s',[4]);
    JFEM=fscanf(fid1,'%g\n');
    basu=fscanf(fid1,'%s',[4]);
    JXFEM=fscanf(fid1,'%g\n');
    basu=fscanf(fid1,'%s',[4]);
    JSUMA=fscanf(fid1,'%g\n');
    for jj=1:9
      tline = fgetl(fid1); 
    end
    basu=fscanf(fid1,'%s',[3]);
    KI=fscanf(fid1,'%g\n');
    basu=fscanf(fid1,'%s',[3]);
    KII=fscanf(fid1,'%g\n');
    basu=fscanf(fid1,'%s',[3]);
    thetaMCS=fscanf(fid1,'%g\n');

    % Escritura
    fprintf(fid2,'%i     %3.15f   %3.15f   %3.15f   %3.15f  %3.15f  %3.15f  %3.15f  %3.15f\n',propag,XYGr(npg,1),XYGr(npg,2),JFEM,JXFEM,JSUMA,KI,KII,thetaMCS);  
     
    fclose(fid1);
    fclose(fid2);
   

end
 
end    