function filegenAbqMC(dibujomalla,caso,propag,meshgenera,numoverlay)
%           Programa Generador de Archivos XFEM para ABAQUS
% ============================================================
%  Crea los archivos para la rutina XFEM:
%   infoXFEM
%   ixfem   %%% No hace falta para ABAQUS
%   BCX     %%% Lo cambio para ABAQUS
%   FORCEX  %%% No hace falta para ABAQUS
%   TOP
%   XY
%=============================================================

% Leer dimensiones placa y NelX, NelY y XYG del archivo input

fid = fopen('input','rt');
tline = fgets(fid);

NelmX = fscanf(fid,'%g\n',[1]);
tline = fgets(fid);

NelmY= fscanf(fid,'%g\n',[1]);
tline = fgets(fid);

w= fscanf(fid,'%g\n',[1]);
tline = fgets(fid);

l= fscanf(fid,'%g\n',[1]);
tline = fgets(fid);

CargasL= fscanf(fid,'%g %g\n',[2]);
tline = fgets(fid);

NumGrietas=fscanf(fid,'%g\n',1);
for ig=1:NumGrietas
    NumPGs(ig)=fscanf(fid,'%g\n',1);   
    XYG= fscanf(fid,'%g %g\n',[2,NumPGs(ig)]);
    XYG=XYG';
    XYGrietas(:,1:NumPGs(ig),ig)=XYG';
end
fclose(fid);


% Generador de la Malla

if meshgenera==0
  fid=fopen('..\InputABQ\m0XY.prn','r');
  XY=fscanf(fid,'%g%*1c    %g%*1c    %g\n',[3,inf]);
  fclose(fid);
  XY=XY';
  fid=fopen('..\InputABQ\m0Top.prn','r');
  Top=fscanf(fid,'%g%*1c    %g%*1c    %g%*1c    %g%*1c    %g\n',[5,inf]);
  fclose(fid);
  Top=Top';
  N=size(XY,1);
  Nel=size(Top,1);
  NNpe=size(Top,2)-1;
  
fid = fopen('XY','w'); 
fprintf(fid,'%3.0f,  %0.8e,  %0.8e\n',XY'); 
fclose(fid);    

fid = fopen('Top','w'); 
fprintf(fid,'%4.0f,  %4.0f,  %4.0f,  %4.0f,  %4.0f\n',Top'); 
fclose(fid);    
    
else  
  [N,Nel,XY,Top,NNpe]=meshgenAbq(NelmX,NelmY,w,l,0);
end 

% --- Busqueda de los nodos del contorno
TopL=[Top(:,2:5) Top(:,2)];
NodCont=zeros(N,1);
ConNodEl=zeros(N,10);
for Ie=1:Nel
    for i=1:NNpe
        ConNodEl(TopL(Ie,i),1)=ConNodEl(TopL(Ie,i),1)+1;
        ConNodEl(TopL(Ie,i),ConNodEl(TopL(Ie,i),1)+1)=Ie;
    end
end
for Ie=1:Nel
    ConEL=zeros(NNpe,1);
    for i=1:NNpe
        n1=TopL(Ie,i);
        n2=TopL(Ie,i+1);
        for j=2:ConNodEl(n1,1)+1
          if ConNodEl(n1,j)~=Ie
             for k=2:ConNodEl(n2,1)+1
                 if ConNodEl(n2,k)==ConNodEl(n1,j)
                    ConEL(i)=ConNodEl(n1,j);
                    continue
                 end
             end
             if ConEL(i)~=0
                 continue
             end
          end
        end
    end
    v=find(ConEL==0);
    if size(v,1)>0
        NodCont(TopL(Ie,v))=1; % El nodo esta en el contorno del dominio
        NodCont(TopL(Ie,v+1))=1;
    end
end
% ---


% Algoritmo LSM
NumEG= 2;
Eps= 1e-6;
integra=1;

% Lazo para cada grieta
VdisNG=[];
MdisNEG=[];
gdln=zeros(N,1);
gdltot=2*N;
ElemGG=[];
NelG=0;
ElemEG=[];
for ig=1:NumGrietas
  XYG=XYGrietas(1:2,1:NumPGs(ig),ig)';
  NumPG= size(XYG,1);
  %[VdisNG MdisNEG gdln gdltot ElemGG NelG ElemFG]= LSM(N,Nel,XY,Top,NNpe,NumPG,XYG,NumEG,Eps,integra);
  [VdisNG0 MdisNEG0 gdln0 gdltot0 ElemGG0 NelG0 ElemEG0]= LSMAbqMC(N,Nel,XY,Top,NNpe,NumPG,XYG,NumEG,Eps,integra,NodCont);
  VdisNG=[VdisNG VdisNG0];
  MdisNEG=[MdisNEG MdisNEG0];
  gdln=max(gdln,gdln0(:,1));
  gdltot=gdltot+gdltot0-2*N;
  ElemGG=[ElemGG; ElemGG0];
  NelG=NelG+NelG0;
  ElemEG=[ElemEG; ElemEG0 ig*ones(size(ElemEG0,1),1)];
end
[xx,ii]=min(VdisNG.^2+MdisNEG.^2,[],2);
VdisNG0=zeros(N,1);
MdisNEG0=zeros(N,1);
for i=1:N
    VdisNG0(i)=VdisNG(i,ii(i));
    MdisNEG0(i)=MdisNEG(i,ii(i));
end
VdisNG=VdisNG0;
MdisNEG=MdisNEG0;
% Fusin de ElemGG y ElemEG (Eugenio)

ElemFusion=[];
contador=1;
for egg=1:size(ElemGG,1)
    ElemFusion=[ElemFusion; ElemGG(egg,:) 0 0];
    if (contador<=size(ElemEG,1))
        if ElemFusion(egg,1)== ElemEG(contador,1)
            ElemFusion(egg,9)=ElemEG(contador,3);
            ElemFusion(egg,10)=ElemEG(contador,2);
            contador=contador+1;
        end
    end 
end
ElemGG=ElemFusion;

% Para que ElemGG quede ordenada con nmero de elemento ascendente, como la
% matriz de topologa TopConX

ElemGG=sortrows(ElemGG,1);


%Para dibujar los gdl enriquecidos: (Eugenio)

if (dibujomalla==1)
   grafgdln_ParaPaper(gdln(:,1),XYGrietas,NumPGs,caso,propag);
%   grafgdln(gdln(:,1),XYGrietas,NumPGs,caso,propag);
   nombrefigura=['malla',num2str(caso-1),'_',num2str(propag)];  
   print('-depsc',nombrefigura)  

end 




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%   Escribir el archivo infoxfem 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

% Escribir el archivo infoxfem

NelementosX=size(ElemGG,1);
NodosX=[];
for i=1:NelementosX
    Ie=ElemGG(i,1);
%    NodosXe=Top(Ie,4:7);
    NodosXe=Top(Ie,2:5);
    NodosX=union(NodosX,NodosXe);
end    
NnodX=size(NodosX,2);

fid = fopen('infoXFEM','w');
fprintf(fid,'!NGr NPG NelmX NnodX NelmEG\n');
fprintf(fid,'%3.0f  %3.0f  %3.0f  %3.0f  %3.0f\n\n',NumGrietas,max(NumPGs),NelementosX,NnodX,size(ElemEG,1));

fprintf(fid,'!XYG\n');
for ig=1:NumGrietas
    XYGrieta=XYGrietas(1:2,1:NumPGs(ig),ig);
    fprintf(fid,'%d\n',NumPGs(ig));
    fprintf(fid,'%0.8e %0.8e\n',XYGrieta');    
end
fprintf(fid,'\n\n');

% Eug: Lo compacto con el siguiente

% Distancias de los nodos X a grieta (Vdist) y a normales en extremo de grieta (Mdist) 

% fprintf(fid,'!Nodo        Vdist	  Mdist\n');
% for i=1:NnodX
%     n=NodosX(i);
%     %linea=sprintf('%3.0f      %0.12e       %0.12e\n',n,VdisNG(n),MdisNEG(n));
%     %fprintf(fid,strrep(linea,'e','*1.e'));
%     fprintf(fid,'%3.0f      %0.8e       %0.8e\n',n,VdisNG(n),MdisNEG(n));
% end
% fprintf(fid,'\n\n');




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555 
%  Para generar ficheros de infoXfem "troceados" y sin cabeceras
%  ver tb. mas adelante
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555

fidnodoXGG = fopen('GGnodeX','w');



% Imprime el tipo de nodo (0 si tiene 2 gdl/nodo, 1 si tiene 4 gdl/nodo o 2 si tiene 10 gdl/nodo) 
% Ademas crea vectores con los nodos 2, 4, o 10 gdl para luego restringir
% comodamente


i2=1;
i4=1;
i10=1;


fprintf(fid,'!Nodo	TipoX        Vdist	  Mdist\n');
for i=1:NnodX
    n=NodosX(i);
    switch(gdln(n,1))
        case 2
            x=0;
            nx2gdl(i2)=n;
            i2=i2+1;
        case 4
            x=1;
            nx4gdl(i4)=n;
            i4=i4+1;
        case 10
            x=2;
            nx10gdl(i10)=n;
            i10=i10+1;
    end
    matrizTipoX(i,1)=n;
    matrizTipoX(i,2)=x;
    fprintf(fid,'%3.0f  %3.0f      %0.8e       %0.8e\n',n,x,VdisNG(n),MdisNEG(n));
    fprintf(fidnodoXGG,'%3.0f  %3.0f      %0.8e       %0.8e\n',n,x,VdisNG(n),MdisNEG(n));
end
fprintf(fid,'\n\n');

fclose(fidnodoXGG);


fprintf(fid,'!ElemGG\n');
%linea=sprintf('%3.0f  %3.0f  %0.12e  %0.12e  %0.12e  %0.12e  %0.12e  %0.12e\n',ElemGG');
%fprintf(fid,strrep(linea,'e','*1.e'));
fprintf(fid,'%3.0f  %3.0f  %0.8e  %0.8e  %0.8e  %0.8e  %0.8e  %0.8e  %3.0f  %3.0f\n',ElemGG');
fprintf(fid,'\n\n');

fprintf(fid,'!ElemEG\n');
fprintf(fid,'%3.0f  %3.0f  %3.0f\n',ElemEG');
fprintf(fid,'\n\n');
fclose(fid);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555 
%  Para generar ficheros de infoXfem "troceados" y sin cabeceras
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555


fidinfoXGG = fopen('GGinfoX','w');
%fprintf(fidinfoXGG,'%3.0f  %3.0f  %3.0f  %3.0f  %3.0f\n',NumGrietas,max(NumPGs),NelementosX,NnodX,size(ElemEG,1));
fprintf(fidinfoXGG,'%3.0f  %3.0f  %3.0f  %3.0f\n',NumGrietas,max(NumPGs),NelementosX,NnodX);
fclose(fidinfoXGG);

fidXYGGG = fopen('GGXYC','w');
for ig=1:NumGrietas
    XYGrieta=XYGrietas(1:2,1:NumPGs(ig),ig);
    fprintf(fidXYGGG,'%d\n',NumPGs(ig));
    fprintf(fidXYGGG,'%0.8e %0.8e\n',XYGrieta);    
end
fclose(fidXYGGG);


fidelemXGG = fopen('GGelemX','w');
fprintf(fidelemXGG,'%3.0f  %3.0f  %0.8e  %0.8e  %0.8e  %0.8e  %0.8e  %0.8e  %3.0f  %3.0f\n',ElemGG');
fclose(fidelemXGG);

% fidelemXEG = fopen('GGelemXEG','w');
% fprintf(fidelemXEG,'%3.0f  %3.0f  %3.0f\n',ElemEG');
% fclose(fidelemXEG);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555 
%  Para generar los NSETS para ABAQUS segun los nodos a enriquecer
%  y luego restringir los gdl no afectados:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555


fidNodoX2gdl=fopen('SETNodeX2dof','w');
fprintf(fidNodoX2gdl,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',nx2gdl'); 
fclose(fidNodoX2gdl);


fidNodoX4gdl=fopen('SETNodeX4dof','w');
fprintf(fidNodoX4gdl,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',nx4gdl'); 
fclose(fidNodoX4gdl);

fidNodoX10gdl=fopen('SETNodeX10dof','w');
fprintf(fidNodoX10gdl,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',nx10gdl'); 
fclose(fidNodoX10gdl);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555 
%  Generacin de tablas NodoX - fila  y ElemGG - fila
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555

% PARA NodoX-FIla

NodoXFila=zeros(N,2);
for i=1:N
    NodoXFila(i,1)=i;
end      
for i=1:NnodX
    n=NodosX(i); 
    NodoXFila(n,2)=i;    
end  

fid = fopen('FilaNodoX','w'); 
fprintf(fid,'%5.0f %5.0f\n',NodoXFila');
fclose(fid); 



% PARA ElemX-Fila

ElemXFila=zeros(Nel,2);
for i=1:Nel
    ElemXFila(i,1)=i;
end      
for i=1:NelementosX
    e=ElemGG(i,1); 
    ElemXFila(e,2)=i;    
end  

fid = fopen('FilaElemX','w'); 
fprintf(fid,'%5.0f %5.0f\n',ElemXFila');
fclose(fid); 


 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555 
%  Generacin de Elem Sets para elementos usuario de ABAQUS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555

% flagU1=0;
% 
% 
% for e=1:NelementosX
%   numel=ElemGG(e,1);
%   tipoel=ElemGG(e,2);
%   if (tipoel~=-1)
%   
%     if (tipoel==1)
%        if (flagU1==0)
%          fidU1=fopen('./meshXFEM/U1','w');
%          fidU1Hpos=fopen('./meshXFEM/U1Hpos','w');
%          fidU1Hneg=fopen('./meshXFEM/U1Hneg','w');
%          flagU1=1;
%        end
%        lado=ElemGG(e,3);
%        NodosXe=Top(numel,2:5);
% %         Para que los nodos sean los dos ltimos
%        if (lado==1)
%          numpermu=2;
%        elseif (lado==2)    
%          numpermu=1;
%        elseif (lado==4)
%          numpermu=3;
%        elseif (lado==3)
%          numpermu=0;
%        end
%        for permu=1:numpermu
%             NodosXe(5)=NodosXe(4); 
%             NodosXe(4)=NodosXe(3);
%             NodosXe(3)=NodosXe(2);
%             NodosXe(2)=NodosXe(1);
%             NodosXe(1)=NodosXe(5);
%        end   
%        fprintf(fidU1,'%5.0f, %5.0f, %5.0f, %5.0f, %5.0f\n',numel,NodosXe(1:4)); 
% %      --- Para saber si est por el lado de distancias positivas o no
%        sumadist=0;
%        for (nn=1:4)
% %          sumadist=sumadist+VdisNG(NodoXFila(NodosXe(nn),2),2); 
%           sumadist=sumadist+VdisNG(NodosXe(nn)); 
%        end
%        if (sumadist<0)
%          fprintf(fidU1Hneg,'%5.0f\n',numel); 
%        elseif (sumadist>0)
%          fprintf(fidU1Hpos,'%5.0f\n',numel);
%        else
%          fprintf('Problema con las distancias a la grieta\n');
%        end                             
%     end 
%   
%   
%   end
% end


%fclose('all');


 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555 
%  Para quitar de la topologa top los elementos que son X y generar los elem overlay:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555



fidTopSinX=fopen('TopNoX','w');
fidTopConX=fopen('TopX','w');
fidTopConXoverlay=fopen('TopXoverlay','w');

for e=1:Nel
  NumeroEl=Top(e,1);
  NodosEl=Top(e,2:5); 
  filaElemGG=ElemXFila(NumeroEl,2);
  tipoel=0;
  if (filaElemGG==0)
    fprintf(fidTopSinX,'%6.0f,%6.0f,%6.0f,%6.0f,%6.0f\n',NumeroEl,NodosEl(1:4));      
  else    
    fprintf(fidTopConX,'%6.0f,%6.0f,%6.0f,%6.0f,%6.0f\n',NumeroEl,NodosEl(1:4)); 
    fprintf(fidTopConXoverlay,'%6.0f,%6.0f,%6.0f,%6.0f,%6.0f\n',NumeroEl+numoverlay,NodosEl(1:4)); 
  end
end

fclose(fidTopSinX);
fclose(fidTopConX);
fclose(fidTopConXoverlay);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555 
%  Para crear una tabla de conectividades, donde el numero de nodo sea sustituido por su tipoX:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%555555555555




fidTopConXTipoX=fopen('TopXTypeX','w');

for i=1:NelementosX
    Ie=ElemGG(i,1);
    NodosXe=Top(Ie,2:5);
    for j=1:4
      TipoXe(j)=matrizTipoX(NodoXFila(NodosXe(j),2),2);   
    end    
    fprintf(fidTopConXTipoX,'%5.0f  %5.0f  %5.0f  %5.0f  %5.0f\n',Ie,TipoXe(1:4));    
end    


fclose(fidTopConXTipoX);





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5555 
% --- Para ABAQUS: generacin de otros elsets y nsets tpicos del problema 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5555
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5555

tol=1e-13;

%X0=0; 
%X1=w; 
%Y0=0; 
%Y1=l; 

minimos=min(XY);
maximos=max(XY);

X0=minimos(2); %La primera columna de XY guarda los nmeros de nodos que no interesan ahora 
X1=maximos(2); 
Y0=minimos(3); 
Y1=maximos(3); 




% Generacin de sets para restricciones
jizq=1;
jder=1;
jaba=1;
jarr=1;

nodocen=0;

iflag=0;
fflag=0;

for i=1:N
  if abs(XY(i,2)-X0)<tol
    ladoizq(jizq)=XY(i,1);
    jizq=jizq+1;
  elseif abs(XY(i,2)-X1)<tol
    ladoder(jder)=XY(i,1);
    jder=jder+1;
  end    

  if abs(XY(i,3)-Y0)<tol
    ladoaba(jaba)=XY(i,1);
    jaba=jaba+1;
  elseif abs(XY(i,3)-Y1)<tol
    ladoarr(jarr)=XY(i,1);
    jarr=jarr+1;
  end    

  if abs(XY(i,2)-X0)<tol & abs(XY(i,3)-Y0)<tol
    nodoabaizq=XY(i,1);
  elseif abs(XY(i,2)-X1)<tol & abs(XY(i,3)-Y0)<tol
    nodoabadch=XY(i,1);
  elseif abs(XY(i,2)-X1)<tol & abs(XY(i,3)-Y1)<tol
    nodoarrdch=XY(i,1);
  elseif abs(XY(i,2)-X0)<tol & abs(XY(i,3)-Y1)<tol
    nodoarrizq=XY(i,1);
  end


  if abs(XY(i,2)-X1)<tol & abs(XY(i,3)-Y1/2)<tol
    nodocen=XY(i,1);
  end
  if abs(XY(i,2)-XYG(1,1))<tol & abs(XY(i,3)-XYG(1,2))<tol
    nodocen=XY(i,1);
  end
  if abs(XY(i,2)-XYG(1,1))<tol & abs(XY(i,3)-XYG(1,2))<tol
    igr=XY(i,1);
    iflag=1;
  elseif (iflag==0)
    igr=0;
  end
  if abs(XY(i,2)-XYG(2,1))<tol & abs(XY(i,3)-XYG(2,2))<tol
    fgr=XY(i,1);
    fflag=1;
  elseif (fflag==0);
    fgr=0;
  end

end

%if (igr==0)
%    fprintf('Cuidado con igrieta. La gr. no empieza en nodo. Revisar en FileGenAbq.m\n')
%    fprintf('En principio est solucionado. No hace falta revisar.\n')  
%end

%if (fgr==0)
%    fprintf('Cuidado con fgrieta. La gr. no acaba en nodo. Revisar en FileGenAbq.m\n') 
%    fprintf('En principio est solucionado. No hace falta revisar.\n')  
%end

% Para restringir en direccin 1 el lado izq, si hay algn nodo enriquecido en ese lado, deber restringirse
% tambin los gdl correspondientes:


contador=1;
for i=1:size(ladoizq,2)
  for j=1:size(nx4gdl,2)
    if ladoizq(i)==nx4gdl(j)
       restrgdl3(contador)=ladoizq(i);
       contador=contador+ 1;
    end
  end
end

% Escritura en fichero

% fid = fopen('SETrestrgdl3','w'); 
% fprintf(fid,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',restrgdl3'); 
% fclose(fid);    




% Elementos cara inferior y superior

for i=1:NelmX
  elemaba(i)=i;
  elemarr(i)=Nel-NelmX+i;
end

% Escritura en ficheros

fid = fopen('SETladoizq','w'); 
fprintf(fid,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',ladoizq'); 
fclose(fid);    

fid = fopen('SETladoder','w'); 
fprintf(fid,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',ladoder'); 
fclose(fid);    

fid = fopen('SETladoarr','w'); 
fprintf(fid,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',ladoarr'); 
fclose(fid);    

fid = fopen('SETladoaba','w'); 
fprintf(fid,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',ladoaba'); 
fclose(fid);    

fid = fopen('SETnodoabaizq','w'); 
fprintf(fid,'%3.0f',nodoabaizq); 
fclose(fid);    

fid = fopen('SETnodoabadch','w'); 
fprintf(fid,'%3.0f',nodoabadch); 
fclose(fid);    

fid = fopen('SETnodoarrdch','w'); 
fprintf(fid,'%3.0f',nodoarrdch); 
fclose(fid);    

fid = fopen('SETnodoarrizq','w'); 
fprintf(fid,'%3.0f',nodoarrizq); 
fclose(fid);    



fid = fopen('SETnodocen','w'); 
fprintf(fid,'%3.0f',nodocen); 
fclose(fid);    

fid = fopen('SETigrieta','w'); 
fprintf(fid,'%3.0f',igr); 
fclose(fid);    

fid = fopen('SETfgrieta','w'); 
fprintf(fid,'%3.0f',fgr); 
fclose(fid);    

fid = fopen('SETelemaba','w'); 
fprintf(fid,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',elemaba'); 
fclose(fid);    

fid = fopen('SETelemarr','w'); 
fprintf(fid,'%3.0f,  %3.0f,  %3.0f, %3.0f, %3.0f, %3.0f, %3.0f, %3.0f \n',elemarr'); 
fclose(fid);    




