function [VdisNG, MdisNEG, gdln, gdltot, ElemGG, NelG, ElemEG] = LSMAbqMC(N,Nel,XY,Top,NNpe,NumPG,XYG,NumEG,Eps,integra,NodCont);
% Calculo inicial de funciones de grieta en nodos
%=============================================================================
% N             : Numero de nodos
% Nel           : Numero de elementos
% XY(N,2)       : Coordenadas de nodos
% Top(Nel,NNpe) : Conectividades de elementos
% NNpe          : Numero de nodos por elemento
% NumPG         : Numero de puntos de definicion de la grieta
% XYG           : Coordenadas de puntos de definicion de la grieta
% NumEG         : Numero de extremos de grieta (1, 2)
% NumEG         : Numero de extremos de grieta a considerar
% Eps           : Tolerancia de distancia
%% VNorTG       : Vector normal en el tramo de grieta
%% VNorPG       : Vector normal en puntos de grieta
% VdisNG(N,1)   : Distancias de nodos a grieta
% MdisNEG(N,1)  : Distancias de nodos a normales en extremo de grieta
% gdln(N,2)     : Matriz de definicion de gdl en nodos
% gdltot        : Numero de gdl totales
% NelG          : Numero de elementos X 
% ElemGG(NelG,8): Definicion de elementos X
%   C1: Numero de elemento
%   C2: Tipo :-1  Elemento no intersectado con grieta 
%              1  Interseccion en 2 nodos contiguos (grieta = lado)
%              2  Interseccion en 2 nodos opuestos
%              3  Interseccion en nodo y lado
%              4  Interseccion en 2 lados opuestos
%              5  Interseccion en 2 lados contiguos
%                 Elementos Especiales Fin de Grieta. Se conservan
%                 caracteristicas tipo de los elementos anteriores.
%             11  Interseccion en 2 nodos contiguos FG en medio
%             12  Interseccion en 2 nodos contiguos FG en Nodo
%             20  Interseccion en 2 nodos opuestos FG dentro del elemento
%             22  Interseccion en 2 nodos opuestos FG en nodos
%             30  Interseccion en nodo y lado FG dentro del elemento
%             31  Interseccion en nodo y lado FG en Lado
%             32  Interseccion en nodo y lado FG en Nodo
%             40  Interseccion en 2 lados opuestos FG dentro del elemento
%             41  Interseccion en 2 lados opuestos FG en Lados
%             50  Interseccion en 2 lados contiguos FG dentro del elemento
%             51  Interseccion en 2 lados contiguos FG en Lado
%
%
%  Tipo    C3      C4      C5      C6      C7      C8
%   -1    
%    1    lado     
%    2     n1      n2
%    3     n1     lado     xi      yi
%    4    lado1   lado2    xi1     yi1     xi2     yi2
%    5    lado1   lado2    xi1     yi1     xi2     yi2
%    6    n1
%    5    lado1   xi1     yi1     
%  (lados y nodos numeracion local -1..4-)
%=============================================================================
%
%XY=XY(:,3:4);
%Top=Top(:,4:7); 
%
% Eug: Cambio para ficheros preparados para ABAQUS
XY=XY(:,2:3); 
Top=Top(:,2:5); 


maxlong= 2*sqrt((max(XY(:,1))-min(XY(:,1)))^2+(max(XY(:,2))-min(XY(:,2)))^2);

% --- Calculo de vectores normales en tramos de grieta

VNorTG= zeros(NumPG-1,2);

for Ipg= 1:NumPG-1;
    x0= XYG(Ipg,1);
    y0= XYG(Ipg,2);
    x1= XYG(Ipg+1,1);
    y1= XYG(Ipg+1,2);
    long= sqrt((x1-x0)^2+(y1-y0)^2);
    VNorTG(Ipg,1)= (y0-y1)/long;
    VNorTG(Ipg,2)= (x1-x0)/long;
end

% --- Calculo de vectores normales en puntos de grieta
% --- (no modulo unitario)

VNorPG= zeros(NumPG,2);

% El tramo tiene 2 puntos de grieta:

VNorPG(1,:)= VNorTG(1,:);
VNorPG(NumPG,:)= VNorTG(NumPG-1,:);

% El tramo tiene ms de 2 puntos de grieta:

for Ipg= 2:NumPG-1
    VNorPG(Ipg,:)= VNorTG(Ipg-1,:) + VNorTG(Ipg,:);
end

% --- Calculo de distancia de nodos a grieta:

VdisNG= ones(N,1)*maxlong;
for In= 1:N
    xn= XY(In,1);
    yn= XY(In,2);
  
  
  % --- calculo de distancias a grieta
  for Itg= 1:NumPG-1
    n= VNorTG(Itg,:);
    n1= VNorPG(Itg,:);
    n2= VNorPG(Itg+1,:);
    xyp1= XYG(Itg,:);
    xyp2= XYG(Itg+1,:);
    flag01=  (n(2)*(xn-xyp1(1))-n(1)*(yn-xyp1(2)));
    flag02= -(n(2)*(xn-xyp2(1))-n(1)*(yn-xyp2(2)));
    dist0= n(1)*(xn-xyp1(1))+n(2)*(yn-xyp1(2));
    signo= sign(dist0);
    flag1=  (n1(2)*(xn-xyp1(1))-n1(1)*(yn-xyp1(2)));
    flag2= -(n2(2)*(xn-xyp2(1))-n2(1)*(yn-xyp2(2)));
    Enc= 0;
    if ((flag01>=0)|(Itg==1))&((flag02>=0)|(Itg==(NumPG-1)))
      dist= dist0;
      Enc= 1;
    else
      if (flag1>=0)&(flag01<=0)
        dist= sqrt((xyp1(1)-xn)^2+(xyp1(2)-yn)^2)*signo;
        Enc= 1;
      else
        if (flag2>=0)&(flag02<=0)
          dist= sqrt((xyp2(1)-xn)^2+(xyp2(2)-yn)^2)*signo;
          Enc= 1;
        end
      end
    end
    if Enc==1 
      if abs(dist)<abs(VdisNG(In))
        if abs(dist)>Eps
          VdisNG(In)= dist;
        else
          VdisNG(In)= 0;
        end
      end
    end
  end
end

% --- Calculo de distancias de nodos a normales en extremo de grieta

t1= XYG(1,:)-XYG(2,:);
long= sqrt(t1(1)^2+t1(2)^2);
t1= t1/long;
t2= XYG(NumPG,:)-XYG(NumPG-1,:);
long= sqrt(t2(1)^2+t2(2)^2);
t2= t2/long;

MdisNEG= zeros(N,1);
NumEGN=zeros(N,1);
for In= 1:N
  xn= XY(In,1);
  yn= XY(In,2);
  MdisNEG(In,1)= (xn-XYG(NumPG,1))*t2(1)+(yn-XYG(NumPG,2))*t2(2);
  NumEGN(In)=NumPG;
  if NumEG > 1
    dist= (xn-XYG(1,1))*t1(1)+(yn-XYG(1,2))*t1(2);
    %Cambio la siguiente linea por el if (Jose)
    %MdisNEG(In,1)= max(MdisNEG(In,1),dist)
    if dist>MdisNEG(In,1)
        MdisNEG(In,1)=dist;
        NumEGN(In)=1;
    end
    
  end
  if abs(MdisNEG(In,1))<Eps
    MdisNEG(In,1)= 0;
  end
end



% --- Definicion de gdl en nodos y elementos X

gdln= zeros(N,2);
gdln(:,1)= 2*ones(N,1);
%ElemGG= zeros(1,1);
%NelG= 0;

%**********************************************************************
ElemFG=[];
reeval=[];
for Ie= 1:Nel
  
    
%______________________________________________    
%______________________________________________    
%********** SOLO PARA FRETTING ****************
% Modificacin para problema de Eugenio (slo enriquece para el specimen
% es decir, para y<0)
%%%% Las siguientes lineas las comenta Jose
% %   if any(XY(Top(Ie,:),2)>0)
% %       continue
% %   end    
%________________________________________________      
    
 
  for j=1: NNpe
    dg(j)= VdisNG(Top(Ie,j));
    dng(j)= MdisNEG(Top(Ie,j),1);
  end
  dmin= min(dg);
  dmax= max(dg);
  dminn= min(dng);
  dmaxn= max(dng);
  
  %Asignar elemento de 4 gdl en nodos           DEBUG
  if dmin*dmax<=0 & all(dng <= 0)            
      if dmin*dmax==0
          for j=1:NNpe
              if dg(j)==0 & gdln(Top(Ie,j),1)==2
                  gdln(Top(Ie,j),1)= 4;
              end
          end    
      else
          for j=1:NNpe
              if gdln(Top(Ie,j),1)==2
                  gdln(Top(Ie,j),1)= 4;
              end
          end    
      end
  end

  if dmin*dmax<=0 & dminn*dmaxn<=0

      %Determinante de Areas para comprobar elemento de Fin de Grieta
      flagf=0;
      flagc=0;

      for i2=1:3
          A=[XYG(NumEGN(Top(Ie,1)),1) XYG(NumEGN(Top(Ie,1)),2) 1;
              XY(Top(Ie,i2),1) XY(Top(Ie,i2),2) 1
              XY(Top(Ie,i2+1),1) XY(Top(Ie,i2+1),2) 1];
          valor(i2)=det(A);
          if valor(i2)<-Eps
              flagf=1;
              reeval=[reeval Ie];
          end
          if valor(i2)<Eps     %si: esta en el contorno
              flagc=1;
              if NodCont(Top(Ie,i2))==1 && NodCont(Top(Ie,i2+1))==1
                  flagf=2; % El EG est en el contorno del dominio
              end
          end
      end
      A=[XYG(NumEGN(Top(Ie,1)),1) XYG(NumEGN(Top(Ie,1)),2) 1;
          XY(Top(Ie,4),1) XY(Top(Ie,4),2) 1
          XY(Top(Ie,1),1) XY(Top(Ie,1),2) 1] ;
      valor(4)=det(A);
      if valor(4)<-Eps
          flagf=1;
          reeval=[reeval Ie];
      end
      if valor(4)<Eps         %si: esta en el contorno
          flagc=1;
          if NodCont(Top(Ie,4))==1 && NodCont(Top(Ie,1))==1
              flagf=2; % El EG est en el contorno del dominio
          end
      end

      %Asignacion de Elemento de Fin de Grieta
      if flagf==0
          for j=1:NNpe
              gdln(Top(Ie,j),1)=10;
              %if XYG(NumPG,:)==XY(Top(Ie,j),:)                              %tengoelementode fin degrieta
              if all(abs(XYG(NumEGN(Top(Ie,1)),:)-XY(Top(Ie,j),:))<Eps)
                  flagc=2;
              end
          end
      end
      
     %Comprobacin de enriquecimiento de Heaviside                  DEBUG
     if flagf==1
       if dng(find(dg==dmax))<=0 & dng(find(dg==dmin))<=0
             for j=1:NNpe
                 if gdln(Top(Ie,j),1)==2
                     gdln(Top(Ie,j),1)=4;
                 end
             end
         end
     end

% Aado esto (EUGENIO)
     
%      if flagf==1
    if flagf==2         
        if dmin*dmax<0 & dminn*dmaxn<0
            for j=1:NNpe
                if gdln(Top(Ie,j),1)==2
                    gdln(Top(Ie,j),1)=4;
                end
            end
        end
    end    
     
     
     
     
      if flagf==0                              % Elemento de Fin de Grieta con caracteristica
          ElemFG=[ElemFG ; Ie flagc ];
      end
  end
%_________________________________________________________  
%  PAra agregar capa de elementos X en el EG manualmente en elementos
%   if any(Ie==[10048 10049 10050  10051 10152 10253 10252 10251 10250 10149])
%       for j=1:NNpe
%           gdln(Top(Ie,j),1)=10;
%       end
%   end
% %___________________________________________________________
end

% Para asegurar los casos excipciones...Mohamad Sabsabi.. 
for i=1:size(reeval,2)
    AM=gdln(Top(reeval(i),:),1);
    if any(AM==2)& any(AM==10)& any(AM==4)
        M=find(AM==2);
        gdln(Top(reeval(i),M),1)=4;
    end 
end
    
%_________________________________________________________________________

% AGREGAR CAPAS EN ELEMENTOS ENRIQUECIDOS EN EL EXTREMO DE GRIETA

%  POR NUMERO DE CAPAS

% for CapasEG=1:1
%     gdlinicial=gdln(:,1);  %Evita reescritura ciclica
%     for Ie=1:Nel
%         if any(gdlinicial(Top(Ie,:))==10)
%             for j=1:NNpe
%                 gdln(Top(Ie,j),1)= 10;
%             end
%         end
%     end
% end
%**************************************************************************
%_________________________________________________________________________

%  CAPAS CON NODOS MIXTOS
% (Heaviside + F(r,theta)) mediante seleccin simple: si el nodo era H, ahora es H+F

% NUMERO DE CAPAS

% for CapasEG=1:1
%     gdlinicial=gdln(:,1);  %Evita reescritura ciclica
%     for Ie=1:Nel
%         if any(gdlinicial(Top(Ie,:))==10)
%             for j=1:NNpe
%                 if gdln(Top(Ie,j),1)==4|gdln(Top(Ie,j),1)==12
%                     gdln(Top(Ie,j),1)= 12; %NODOS MIXTOS
%                     gdln(Top(Ie,j),1)= 10;
%                 else
%                     gdln(Top(Ie,j),1)= 10;
%                 end    
%             end
%         end
%     end
% end
% %**************************************************************************
%--------------------------------------------------------------------------
%  CAPAS CON UN RADIO RIJO ALREDEDOR DEL EXTREMO DE GRIETA
% Nodos Mixtos en INTERIOR

% radio=0.5;
% for Ie= 1:Nel
%     if any(abs(VdisNG(Top(Ie,:)))<=radio & abs(MdisNEG(Top(Ie,:))<=radio))
%         for i=1:NNpe
%             if sqrt(VdisNG(Top(Ie,i))^2+MdisNEG(Top(Ie,i))^2)<=radio
%                 if gdln(Top(Ie,i),1)==4|gdln(Top(Ie,i),1)==12
% %                     gdln(Top(Ie,i),1)=12;       %NODOS MIXTOS
%                     gdln(Top(Ie,i),1)=10;
%                 else
%                     gdln(Top(Ie,i),1)=10;
%                 end
%             end    
%         end    
%     end  
% end    
%--------------------------------------------------------------------------
%Nodos mixtos en CONTORNO

% radio=0.5;
% gdlinicial=gdln(:,1);
% for Ie= 1:Nel
%     if any(abs(VdisNG(Top(Ie,:)))<=radio & abs(MdisNEG(Top(Ie,:))<=radio))
%         for i=1:NNpe
%             if sqrt(VdisNG(Top(Ie,i))^2+MdisNEG(Top(Ie,i))^2)<=radio
%                 if gdln(Top(Ie,i),1)==4|gdln(Top(Ie,i),1)==12
%                     gdln(Top(Ie,i),1)=10;
%                 else
%                     gdln(Top(Ie,i),1)=10;
%                 end
%             end    
%         end    
%     end  
% end
% for Ie=1:Nel
%     if any(gdln(Top(Ie,:))==10)& any(gdln(Top(Ie,:))==4)
%         for j=1:NNpe
%             if gdln(Top(Ie,j),1)==10 & gdlinicial(Top(Ie,j))==4;
%                 gdln(Top(Ie,j),1)= 10; %--->10 No Mixtos, 12 NMix Contorno
%             end
%         end
%     end
% end    

%__________________________________________________________________________


%Matriz ElemGG  

NelG= 0;
ElemGG= zeros(1,8);     
ElemEG=[]; % Numero de punto extremo de grieta asociado al elemento
for Ie= 1:Nel
  if (gdln(Top(Ie,1))~=2)|(gdln(Top(Ie,2))~=2)|(gdln(Top(Ie,3))~=2)|(gdln(Top(Ie,4))~=2)
    NelG= NelG+1;
    ElemGG(NelG,1)= Ie;
    ElemGG(NelG,2)= -1;
    %gdls=gdln(Top(Ie,1))+gdln(Top(Ie,2))+gdln(Top(Ie,3))+gdln(Top(Ie,4));
    %if gdls == 40 | gdls==28
    %  ElemGG(NelG,9)= -100;
    %end
 end
end

% Numeracion de gdl
% =================

gdln(1,2)=1;
for i=2:N
  gdln(i,2)= gdln(i-1,2) + gdln(i-1,1);
end
gdltot= gdln(N,1)+gdln(N,2)-1;

%__________________________________________________________________
% --- Definicion de intersecciones grieta elementos X
% --- (matriz ElemGG)

for IeG=1: NelG
  
  Ie= ElemGG(IeG,1);
  for j= 1: NNpe
    inod=Top(Ie,j);
    disNGe(j)= VdisNG(inod,1);
    MdisNEGe(j)= MdisNEG(inod,1);
    Xe(j)= XY(inod,1);
    Ye(j)= XY(inod,2);
  end
  % --- calculo de intersecciones grieta-nodos elemento
  NIntNGe= 0;
  for j= 1:NNpe
    if (abs(disNGe(j))<Eps)&((MdisNEGe(j)<=0)|sum(gdln(Top(Ie,:)))>=28)
      NIntNGe= NIntNGe + 1;
      ini(NIntNGe)= j;
    end
  end
  % --- 2 intersecciones con nodos
  Acabar= 0;
  if NIntNGe ==2
    Acabar= 1;
    % - Intersecciones en nodos contiguos
    if (ini(2)-ini(1)== 1);
      ElemGG(IeG,2)= 1;
      ElemGG(IeG,3)= ini(1);
    elseif (ini(1)==1&(ini(2)==4));
      ElemGG(IeG,2)= 1;
      ElemGG(IeG,3)= ini(2);
    % - Intersecciones en nodos opuestos
    else  
      ElemGG(IeG,2)= 2;
      ElemGG(IeG,3)= ini(1);
      ElemGG(IeG,4)= ini(2); 
    end
  end
  % --- Calculo de intersecciones grieta-lados      
  NIntLGe= 0;
  if Acabar==0
    if NIntNGe ==1
      ladpos=2;
      lados(1)= ini(1)+1;
      lados(2)= ini(1)+2;
      if lados(1)> NNpe
        lados(1)= lados(1) -NNpe;
      end
      if lados(2)> NNpe
        lados(2)= lados(2) -NNpe;
      end
    else
      ladpos= 4;
      for j= 1:NNpe
        lados(j)= j;
      end
    end
    for Ilad= 1:ladpos
      val1= VdisNG(Top(Ie,lados(Ilad)));
      valn1= MdisNEG(Top(Ie,lados(Ilad)));
      if lados(Ilad)==4
        val2= VdisNG(Top(Ie,1));
        valn2= MdisNEG(Top(Ie,1));
      else
        val2= VdisNG(Top(Ie,lados(Ilad)+1));
        valn2= MdisNEG(Top(Ie,lados(Ilad)+1));
      end
      if ((val2-val1)~=0)&(((valn1<=0)&(valn2<=0))|(sum(gdln(Top(Ie,:)))>=28))
        s= -val1/(val2-val1);
        if (s>0)&(s<1)
          NIntLGe= NIntLGe+1;
          ili(NIntLGe,1)= lados(Ilad);
          x1= XY(Top(Ie,lados(Ilad)),1);
          y1= XY(Top(Ie,lados(Ilad)),2);
          if lados(Ilad)==4
            x2= XY(Top(Ie,1),1);
            y2= XY(Top(Ie,1),2);
          else
            x2= XY(Top(Ie,lados(Ilad)+1),1);
            y2= XY(Top(Ie,lados(Ilad)+1),2);
          end
          ili(NIntLGe,2)= x1+ s*(x2-x1);
          ili(NIntLGe,3)= y1+ s*(y2-y1);
        end
      end
    end
    
    %- Interseccion nodo lado  
    if (NIntNGe==1)&(NIntLGe==1)
      ElemGG(IeG,2)= 3;
      ElemGG(IeG,3)= ini(1);
      ElemGG(IeG,4)= ili(1,1);
      ElemGG(IeG,5)= ili(1,2);
      ElemGG(IeG,6)= ili(1,3);
    end
    
    %- Interseccion en 2 lados
    if NIntLGe==2
      diff= abs(ili(1,1)-ili(2,1));
      if 2*round(diff/2)==diff
        ElemGG(IeG,2)= 4;
      else
        ElemGG(IeG,2)= 5;
      end
      ElemGG(IeG,3)= ili(1,1);
      ElemGG(IeG,4)= ili(2,1);
      ElemGG(IeG,5)= ili(1,2);
      ElemGG(IeG,6)= ili(1,3);
      ElemGG(IeG,7)= ili(2,2);
      ElemGG(IeG,8)= ili(2,3);
    end      
  end
  
  %_______________________________________________
  %Especificacion de Tipo de Elemento en Fin de Grieta 
  if integra ==1                                            %Integracion simple o modificada
    if size(ElemFG,1)>0
      if any(ElemFG(:,1)==Ie)
         pos=find(ElemFG==Ie);
         ElemGG(IeG,2)=ElemGG(IeG,2)*10+ElemFG(pos,2);
         if ElemGG(IeG,2)==-8                              %revisar casos de ana: -8 ->80
             ElemGG(IeG,2)=80;
         end    
      end  
    end
  end    
  %__________________________________________________
   
  %Especificacion de extremo de grieta asociado al elemento
  nodoF=find(gdln(Top(Ie,:),1)>=10);
  if size(nodoF,1)>0
      ElemEG=[ElemEG; Ie NumEGN(Top(Ie,nodoF(1)))];
  end
   
end
 
%________________________________________________________________________
  